@@ -102,7 +102,7 @@ geneticProgramming <- function(fitnessFunction,
                                mutationFunction = NULL,
                                restartCondition = makeEmptyRestartCondition(),
                                restartStrategy = makeLocalRestartStrategy(),
-                               searchHeuristic = makeAgeFitnessComplexityParetoGpSearchHeuristic(),
+                               searchHeuristic = makeAgeFitnessComplexityParetoGpSearchHeuristic(lambda = ceiling(0.5 * populationSize)),
                                breedingFitness = function(individual) TRUE,
                                breedingTries = 50,
                                extinctionPrevention = FALSE,
