@@ -1,4 +1,4 @@
-## function_utils.r
+## function_utils.R
 ##   - Utility functions for R functions
 ##
 ## RGP - a GP system for R
@@ -13,17 +13,31 @@
 ##'
 ##' @note Always use this function to dynamically generate new functions that are not clojures
 ##' to prevent hard to find memory leaks.
+##' @param envir The new function closure's environment, defaults to \code{globalenv()}.
 ##' @return A new function that does not take any arguments and always returns \code{NULL}.
-new.function <- function() {
+new.function <- function(envir = globalenv()) {
   # The following line has to be inside this function to prevent capturing the lexical
   # environment, which would cause a hard to find memory leak:
   fun <- function() NULL
   # If you use the last line outside of this function, by shure to initialize the
   # environment of the generated function object to the global environment, like done here:
-  environment(fun) <- globalenv() 
+  environment(fun) <- envir
   fun 
 }
 
+##' Create a new R closure given a function body expression and an argument list 
+##'
+##' Creates a R closure (i.e. a function object) from a body expression and an argument
+##' list. The closure's environment will be the default environment.
+##'
+##' @param fbody The function body, given as a R expression.
+##' @param fargs The formal arguments, given as a list or vector of strings.
+##' @param envir The new function closure's environment, defaults to \code{globalenv()}.
+##' @return A formal argument list, ready to be passed via \code{\link{formals}}.
+##' @export
+makeClosure <- function(fbody, fargs, envir = globalenv())
+  .Call("make_closure", fbody, fargs, envir)
+
 ##' Create a new function argument list from a list or vector of strings
 ##'
 ##' Creates a formal argument list from a list or vector of strings, ready to be assigned via
@@ -84,16 +98,13 @@ iterate <- function(n, f, arg, ...) {
 ##' @param f The function to determine the arity for.
 ##' @return The arity of the function \code{f}.
 arity <- function(f) {
-  if (is.primitive(f))
+  if (is.primitive(f)) {
     arity.primitive(f)
-  else if (is.function(f))
+  } else if (is.function(f)) {
     length(formals(f))
-  else if (is.name(f))
-    arity(eval(f))
-  else if (is.character(f))
-    arity(as.name(f))
-  else
+  } else {
     stop("could not determine arity of object \"", f, "\" (of class ", class(f), ")")
+  }
 }
 
 ##' Determine the number of arguments of a primitive function
@@ -146,21 +157,21 @@ arity.primitive <- function(f) {
     stop("could not determine arity of primitive")
 }
 
-##' Tabulate a function
+##' Tabulate an n-ary function
 ##'
-##' Creates a data frame of values for the function \code{f} at the domain points
-##' given in \code{...}. This function works like \code{\link{mapply}}, but returns
-##' a data frame that also contains the domain points instead of a simple vector that
-##' only contains (range) values of \code{f}. When using element names in \code{...},
-##' these names must match the formal parameter names of \code{f}.
+##' Creates a data frame of values for the n-ary function \code{f} at the sample
+##' locations given in \code{...}.
 ##'
 ##' @param f The function to tabulate.
-##' @param ... The points to tabulate \code{f} at.
+##' @param ... For each dimension, a vector of sample points to calculate \code{f} at.
 ##' @return A data frame of function values of \code{f}.
-##' @seealso \code{\link{mapply}}
-##' @export
+##' @export 
 tabulateFunction <- function(f, ...) {
-  xs <- list(...)
-  y <- mapply(f, ...)
-  data.frame(xs, y)
-}
+  xs <- expand.grid(...)
+  colnames(xs) <- NULL
+  ys <- apply(xs, 1, function(args) do.call(f, as.list(args)))
+  functionTable <- cbind(xs, ys)
+  colnames(functionTable) <- c(paste("x", 1:length(list(...)), sep = ""), "y")
+  functionTable
+} 
+
